/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************
 *                                                                           *
 *   FILE: urng_gslqmc.h                                                     *
 *                                                                           *
 *   PURPOSE:                                                                *
 *     Function prototypes for using URNG of type GSL-:                       *
 *     Function prototypes for using uniform of type GSL:                    *
 *     uniform random number from GSL (GNU Scientific Library),              *
 *     see http://www.gnu.org/software/gsl/.                                 *
 *                                                                           *
 *   USAGE:                                                                  *
 *         only included in unuran.h                                         *
 *                                                                           *
 *****************************************************************************
     $Id: urng_gslqrng.h,v 1.1 2005/07/26 14:27:27 leydold Exp $
 *****************************************************************************
 *                                                                           *
 *   Copyright (c) 2000 Wolfgang Hoermann and Josef Leydold                  *
 *   Dept. for Statistics, University of Economics, Vienna, Austria          *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA                  *
 *                                                                           *
 *****************************************************************************/

/*---------------------------------------------------------------------------*/
#ifndef URNG_GSLQRNG_H_SEEN
#define URNG_GSLQRNG_H_SEEN
/*---------------------------------------------------------------------------*/
#if defined(UNURAN_HAS_GSL) && UNUR_URNG_TYPE == UNUR_URNG_GENERIC
/*---------------------------------------------------------------------------*/
#include <gsl/gsl_qrng.h>
/*---------------------------------------------------------------------------*/

/* 
   =NODE  URNG-GSLQRNG  Interface to GSL generators for quasi-random points

   =UP URNG [30]

   =DESCRIPTION
      Interface to the generators for quasi-random points (also called
      low discrepancy point sets) from the GNU Scientific Library (GSL).
      Documentation and source code of this library is available from
      @uref{http://www.gnu.org/software/gsl/}.

   =HOWTOUSE
      This library has to be installed before compiling UNURAN and
      UNURAN_HAS_GSL has to be defined in @file{src/unuran_config.h}.
      Do not forget to link your executables against @file{libgsl}.

      The following routines are supported for URNG objects of
      type GSL-QRNG:

      @itemize @minus
      @item unur_urng_sample()
      @item unur_urng_sample_array()
      @item unur_urng_reset() 
      @item unur_urng_sync() 
      @item unur_urng_free()
      @end itemize

      unur_urng_sync() is used to jump to the first coordinate of
      the next point generated by the generator.

   =END

*/

/*---------------------------------------------------------------------------*/

/* =ROUTINES */

/*---------------------------------------------------------------------------*/

UNUR_URNG *unur_urng_gslqrng_new( const gsl_qrng_type *qrngtype, unsigned int dim );
/*
   Make object for quasi-random point generators for dimension
   @var{dim} from the @file{GSL} (GNU Scientific Library). 
   @var{qrngtype} is the type of the chosen generator as described in
   the GSL manual (see section Quasi-Random Sequences). 
   This library is available from @uref{http://www.gnu.org/software/gsl/}.
*/

/* =END */

/*---------------------------------------------------------------------------*/
#endif  /* defined(UNURAN_HAS_GSL) && UNUR_URNG_TYPE == UNUR_URNG_GENERIC    */
/*---------------------------------------------------------------------------*/
#endif  /* URNG_GSLQRNG_H_SEEN */
/*---------------------------------------------------------------------------*/
